@@ -32,62 +32,62 @@
 #endif
 
 #include "mbfilter.h"
-#include "mbfilter_iso8859_9.h"
-#include "unicode_table_iso8859_9.h"
+#include "mbfilter_iso8859_15.h"
+#include "unicode_table_iso8859_15.h"
 
-static const char *mbfl_encoding_8859_9_aliases[] = {"ISO_8859-9", "latin5", NULL};
+static const char *mbfl_encoding_8859_15_aliases[] = {"ISO_8859-15", NULL};
 
-const mbfl_encoding mbfl_encoding_8859_9 = {
-	mbfl_no_encoding_8859_9,
-	"ISO-8859-9",
-	"ISO-8859-9",
-	(const char *(*)[])&mbfl_encoding_8859_9_aliases,
+const mbfl_encoding mbfl_encoding_8859_15 = {
+	mbfl_no_encoding_8859_15,
+	"ISO-8859-15",
+	"ISO-8859-15",
+	(const char *(*)[])&mbfl_encoding_8859_15_aliases,
 	NULL,
 	MBFL_ENCTYPE_SBCS
 };
 
-const struct mbfl_identify_vtbl vtbl_identify_8859_9 = {
-	mbfl_no_encoding_8859_9,
+const struct mbfl_identify_vtbl vtbl_identify_8859_15 = {
+	mbfl_no_encoding_8859_15,
 	mbfl_filt_ident_common_ctor,
 	mbfl_filt_ident_common_dtor,
 	mbfl_filt_ident_true
 };
 
-const struct mbfl_convert_vtbl vtbl_8859_9_wchar = {
-	mbfl_no_encoding_8859_9,
+const struct mbfl_convert_vtbl vtbl_8859_15_wchar = {
+	mbfl_no_encoding_8859_15,
 	mbfl_no_encoding_wchar,
 	mbfl_filt_conv_common_ctor,
 	mbfl_filt_conv_common_dtor,
-	mbfl_filt_conv_8859_9_wchar,
+	mbfl_filt_conv_8859_15_wchar,
 	mbfl_filt_conv_common_flush
 };
 
-const struct mbfl_convert_vtbl vtbl_wchar_8859_9 = {
+const struct mbfl_convert_vtbl vtbl_wchar_8859_15 = {
 	mbfl_no_encoding_wchar,
-	mbfl_no_encoding_8859_9,
+	mbfl_no_encoding_8859_15,
 	mbfl_filt_conv_common_ctor,
 	mbfl_filt_conv_common_dtor,
-	mbfl_filt_conv_wchar_8859_9,
+	mbfl_filt_conv_wchar_8859_15,
 	mbfl_filt_conv_common_flush
 };
 
 #define CK(statement)	do { if ((statement) < 0) return (-1); } while (0)
 
 /*
- * ISO-8859-9 => wchar
+ * ISO-8859-15 => wchar
  */
-int mbfl_filt_conv_8859_9_wchar(int c, mbfl_convert_filter *filter)
+int mbfl_filt_conv_8859_15_wchar(int c, mbfl_convert_filter *filter)
 {
 	int s;
 
 	if (c >= 0 && c < 0xa0) {
 		s = c;
 	} else if (c >= 0xa0 && c < 0x100) {
-		s = iso8859_9_ucs_table[c - 0xa0];
+		s = iso8859_15_ucs_table[c - 0xa0];
 		if (s <= 0) {
 			s = c;
 			s &= MBFL_WCSPLANE_MASK;
-			s |= MBFL_WCSPLANE_8859_9;
+			s |= MBFL_WCSPLANE_8859_15;
 		}
 	} else {
 		s = c;
@@ -101,9 +101,9 @@ int mbfl_filt_conv_8859_9_wchar(int c, mbfl_convert_filter *filter)
 }
 
 /*
- * wchar => ISO-8859-9
+ * wchar => ISO-8859-15
  */
-int mbfl_filt_conv_wchar_8859_9(int c, mbfl_convert_filter *filter)
+int mbfl_filt_conv_wchar_8859_15(int c, mbfl_convert_filter *filter)
 {
 	int s, n;
 
@@ -113,13 +113,13 @@ int mbfl_filt_conv_wchar_8859_9(int c, mbfl_convert_filter *filter)
 		s = -1;
 		n = 95;
 		while (n >= 0) {
-			if (c == iso8859_9_ucs_table[n]) {
+			if (c == iso8859_15_ucs_table[n]) {
 				s = 0xa0 + n;
 				break;
 			}
 			n--;
 		}
-		if (s <= 0 && (c & ~MBFL_WCSPLANE_MASK) == MBFL_WCSPLANE_8859_9) {
+		if (s <= 0 && (c & ~MBFL_WCSPLANE_MASK) == MBFL_WCSPLANE_8859_15) {
 			s = c & MBFL_WCSPLANE_MASK;
 		}
 	}
@@ -134,5 +134,3 @@ int mbfl_filt_conv_wchar_8859_9(int c, mbfl_convert_filter *filter)
 
 	return c;
 }
-
-
